use editor_state::GraphEditorState;

mod graph_node_ui;

mod color_hex_utils;
mod device;
mod editor_state;
mod graph_editor_egui;
mod graph_impls;
mod graph_types;
mod id_types;
mod index_impls;
mod node_finder;
mod param_ui;
mod virtual_device;

use device::Device;
use virtual_device::VirtualDevice;

#[derive(Default)]
struct App {
    state: GraphEditorState,
    devices: Vec<Device>,
    virtual_device: Option<VirtualDevice>,
}

impl App {
    fn new() -> Self {
        Self {
            state: GraphEditorState::default(),
            devices: Device::query(),
            virtual_device: None,
        }
    }
}

impl epi::App for App {
    fn update(&mut self, ctx: &egui::CtxRef, _frame: &epi::Frame) {
        egui::CentralPanel::default().show(ctx, |_| {
            graph_editor_egui::draw_graph_editor(
                &ctx,
                &mut self.state,
                &self.devices,
                &mut self.virtual_device,
            );
        });
    }

    fn name(&self) -> &str {
        "Device Node Mapper"
    }
}

fn main() {
    let options = eframe::NativeOptions {
        ..Default::default()
    };

    eframe::run_native(Box::new(App::new()), options);
}
